#include "femElementSet.h"
#include "femGmsh.h"

namespace fem
{

ElementSet::ElementSet(int _typ,
                       std::vector<std::size_t> const &_etags,
                       std::vector<std::size_t> const &_ntags) : type_id(_typ),
                                                                 etags(_etags),
                                                                 ntags_v(_ntags),
                                                                 ntags(&ntags_v[0], etags.size(), ntags_v.size() / etags.size())
{

    // properties of type "type_id" (could be shared)
    std::vector<double> localNodeCoord;
    gmsh::model::mesh::getElementProperties(type_id,
                                            type_name, type_dim,
                                            type_order, type_nnods,
                                            localNodeCoord,
                                            type_numPrimaryNodes);

    // localNodeCoord is [xi1, eta1, xi2, eta2, xi3, eta3, ...] for 2d elements
    // which is inconsistent with localCoords returned for the Gauss points
    // [xi1, eta1, 0, xi2, eta2, 0, xi3, eta3, 0, ...]
    type_localNodeCoord.resize(type_nnods * 3);
    std::fill(type_localNodeCoord.begin(), type_localNodeCoord.end(), 0.0);

    for (int i = 0; i < type_nnods; ++i)
        for (int j = 0; j < type_dim; ++j)
            type_localNodeCoord[3 * i + j] = localNodeCoord[type_dim * i + j];
}

ElementSet::~ElementSet()
{
    // std::cout << "~ElementSet()\n";
}

FEM_API std::ostream &
operator<<(std::ostream &out, ElementSet const &obj)
{
    out << "set of " << obj.etags.size() << " \"" << obj.type_name
        << "\" (type=" << obj.type_id << ", order=" << obj.type_order
        << ", nnods=" << obj.type_nnods << ")";
    return out;
}

} // namespace fem
